@@ -139,20 +139,24 @@ sub _build_sql_and_bind {
       # get the value
       my $value = $valuehash->{ $field };
 
-      # should this be "IS NULL" rather than "= ''"
+      # should this be "IS NULL" rather than "= ''"?
       if ($oper eq "=" && !defined($value)) {
         push @conditions, "$field IS NULL";
-      } elsif (!defined($value)) {
-        # just an undef.  I hope $oper is "IS" or "IS NOT"
-        push @conditions, "$field $oper NULL";
-      } else {
-        # proper value, quote it properly
-        # we do this instead of adding to bind because it makes the
-        # error messages much more readable
-        unless ($self->has_dbh)
-          { croak "Needed to quote SQL during SQL building but no 'dbh' defined" }
-        push @conditions, "$field $oper ".$self->dbh->quote($value);
+        next;
       }
+
+      # just an undef?  I hope $oper is "IS" or "IS NOT"
+      if (!defined($value)) {
+        push @conditions, "$field $oper NULL";
+        next;
+      }
+
+      # proper value, quote it properly
+      # we do this instead of adding to bind because it makes the
+      # error messages much more readable
+      unless ($self->has_dbh)
+        { croak "Needed to quote SQL during SQL building but no 'dbh' defined" }
+      push @conditions, "$field $oper ".$self->dbh->quote($value);
     }
   }
 
@@ -180,7 +184,7 @@ sub _build_db_results {
 
   # load "Encode" if we need to do utf8 munging
   if ($self->force_utf8) {
-    eval "use Encode; 1"
+    eval { require Encode; 1 }
       or croak "Can't load Encode, but force_utf8 is enabled";
   }
 
@@ -239,23 +243,30 @@ sub _coerce_and_verify_tests {
 
       if (!defined($value)) {
         $tests->{'eq'}{ $key } = $value;
-      } elsif (ref($value) eq "Regexp") {
-        $tests->{'=~'}{ $key } = $value;
-      } elsif ($value =~ /\A $RE{num}{real} \z/x) {
-        $tests->{'=='}{ $key } = $value;
-      } else {
-        # default to string comparison
-        $tests->{'eq'}{ $key } = $value;
+        next;
       }
+
+      if (ref($value) eq "Regexp") {
+        $tests->{'=~'}{ $key } = $value;
+        next;
+      }
+
+      if ($value =~ /\A $RE{num}{real} \z/x) {
+        $tests->{'=='}{ $key } = $value;
+        next;
+      }
+
+      # default to string comparison
+      $tests->{'eq'}{ $key } = $value;
     }
   }
 
   # check we've got a hash of hashes
   unless (ref($tests) eq "HASH")
-    { croak "Can't understand the argument passed in 'tests'" }
-  foreach my $valuehash (values %{ $tests }) {
-    unless (ref($valuehash) eq "HASH")
-      { croak "Can't understand the argument passed in 'tests'" }
+    { croak "Can't understand the argument passed in 'tests': not a hashref or arrayref" }
+  foreach my $valuekey (keys %{ $tests }) {
+    unless (ref($tests->{ $valuekey }) eq "HASH")
+      { croak "Can't understand the argument passed in 'tests': key '$valuekey' didn't contain a hashref" }
   }
 
   return $tests;
@@ -342,24 +353,34 @@ sub row_at_index_ok {
 
     # process each field in turn, sorted asciibetically
     foreach my $colname (sort keys %{$valuehash}) {
-      # work out what we expect
-      my $expect = $valuehash->{ $colname };
-      my $got    = $data->{ $colname };
 
+      # check the column we're comparing exists
       unless (exists($data->{ $colname })) {
         croak "No column '$colname' returned from table '@{[ $self->table ]}'"
           if $self->has_table;
         croak "No column '$colname' returned from sql";
       }
 
+
+
       # try the comparison
-      unless (do {
+      my $expect = $valuehash->{ $colname };
+      my $got    = $data->{ $colname };
+      my $passed;
+      {
         # disable warnings as we might compare undef
         local $SIG{__WARN__} = sub {}; # $^W not work
 
-        # do a string eval
-        eval "\$got $oper \$expect"
-      }) {
+        # do a string eval because $oper could be any
+        # arbitary comparison operator here.  Note the
+        # the use of backslashes here so that we create
+        # a string containing varaible names *not* the
+        # values.
+        eval "\$passed = \$got $oper \$expect; 1"
+          or croak "Invalid operator test '$oper': $@";
+      };
+
+      unless ($passed) {
         return $self->_fail(
           "While checking column '$colname' on $row_index_th row",
           ( $oper =~ /\A (?:eq|==) \z/x )
@@ -464,19 +485,19 @@ sub _is_diag {
   my($self, $got, $type, $expect) = @_;
 
   foreach my $val (\$got, \$expect) {
-      if( defined ${$val} ) {
-          if( $type eq 'eq' ) {
-              # quote and force string context
-              ${$val} = "'${$val}'"
-          }
-          else {
-              # force numeric context
-              ${$val} = ${$val}+0;
-          }
-      }
-      else {
-          ${$val} = 'NULL';
-      }
+    unless( defined ${$val} ) {
+      ${$val} = 'NULL';
+      next;
+    }
+
+    if( $type eq 'eq' ) {
+      # quote and force string context
+      ${$val} = "'${$val}'";
+      next;
+    }
+
+    # otherwise force numeric context
+    ${$val} = ${$val}+0;
   }
 
   return sprintf <<"DIAGNOSTIC", $got, $expect;
